require "sphere"
require 'pyramid'
require 'cube'

function love.load()
	mouseIsDown = false
	drawEdges = true
	love.graphics.setLineStyle('smooth')
	centerX, centerY = love.graphics.getDimensions()
	centerX = centerX/2
	centerY = centerY/2
	scale = 200
	nodes = pyramid.nodes
	edges = pyramid.edges
end

function love.update(dt)
	if love.mouse.isDown('1') then
		mouseIsDown = true
	else
		mouseIsDown = false
	end

	if love.keyboard.isDown('up') then
		scale = scale + 1
	end
	
	if love.keyboard.isDown('down') then
	 	scale = scale - 1
	end

	if love.keyboard.isDown('1') then
		nodes = pyramid.nodes
		edges = pyramid.edges
	end

	if love.keyboard.isDown('2') then
		nodes = sphere.nodes
		edges = sphere.edges
	end

	if love.keyboard.isDown('3') then
		nodes = cube.nodes
		edges = cube.edges
	end
end

function love.mousemoved(x, y, dx, dy)
	if mouseIsDown == true then
		rotateY3D(dx * 0.01)
    	rotateX3D(dy * 0.01)
    end
end 

function love.draw()
	for i = 1, #nodes, 1 do
		node = nodes[i]
		tf = wcam(node,3)
		pNode = projection(tf,90)
		love.graphics.points(pNode.x*scale+centerX, pNode.y*scale+centerY)
	end

	love.graphics.print(love.timer.getFPS())

	if drawEdges == true then
		for n = 1, #edges, 1 do
			n0 = edges[n][1]
			n1 = edges[n][2]
			node00 = nodes[n0]
			node11 = nodes[n1]
			tf0 = wcam(node00,3)
			tf1 = wcam(node11,3)
			pNode00 = projection(tf0,90)
			pNode11 = projection(tf1,90)
			love.graphics.line(pNode00.x*scale+centerX, pNode00.y*scale+centerY, pNode11.x*scale+centerX, pNode11.y*scale+centerY)
			print(n0)
		end
	end
end

function wcam(v,d)
	t = {}
	t.x = -v.x
	t.y = -v.y
	t.z = d -v.z
	return t
end

function projection(v,fov)
	t = math.tan(math.rad(fov/2))
	n = {}
	w = v.z*-1
	n.x = (v.x*(1/t))/w
	n.y = (v.y*(1/t))/w
	return n
end

function rotateY3D(alpha)
	sin = math.sin(alpha)
	cos = math.cos(alpha)
	for a = 1, #nodes, 1 do
		node = nodes[a]
		x = node.x
		z = node.z
		node.x = (x * cos + z * sin)
		node.z = (z * cos - x * sin)
	end
end

function rotateX3D(alpha)
	sin = math.sin(alpha)
	cos = math.cos(alpha)
	for a = 1, #nodes, 1 do
		node = nodes[a]
		y = node.y
		z = node.z
		node.y = (y * cos + z * sin)
		node.z = (z * cos - y * sin)
	end
end